<?php

namespace Goldenscarab\Modulus\Service\Field\Compose;

use Illuminate\Support\Str;
use \Illuminate\Support\HtmlString;


class FieldBt4
{

    private $type;

    private $label;

    private $id;

    private $class;

    private $name;

    private $value;

    private $placeholder;

    private $prefix;

    private $suffix;

    private $help;

    private $indice;

    private $style;

    private $rows;

    private $min;

    private $max;

    private $size;

    private $is_multiple;

    private $is_autocomplete;

    private $is_autofocus;

    private $is_disabled;

    private $is_required;

    private $is_readonly;

    private $is_inline;

    private $_datas;

    private $_values;

    private $_options;

    private $_popover;

    private $_attributes;


    public function __construct($attrs = [])
    {
        $this->setAttrs($attrs);
    }

    /**
     * Retourne le rendu du champs Code avec ses paramètres
     * @return string Le rendu du champs
     */
    public function renderCodeEditor():string
    {
        $template  = '<div id="editor-%s" class="code-editor"%s>%s</div>' . PHP_EOL;
        $template .= '<textarea id="%s" name="%s" style="display: none;">%s</textarea>';

        $this->_datas['target'] = '#' . $this->id;

        $render = vsprintf($template, [
            $this->id,
            $this->renderFieldAttrs(['style']),
            e($this->getValue()),
            $this->id,
            $this->name,
            e($this->getValue()),
        ]);

        return $render;
    }

    /**
     * Retourne le rendu du champs File avec ses paramètres
     * @return string Le rendu du champs
     */
    public function renderFile(): string
    {
        $template  = '<div class="custom-file">' . PHP_EOL;
        $template .= '    %s' . PHP_EOL;
        $template .= '    <label class="custom-file-label" for="%s">%s</label>' . PHP_EOL;
        $template .= '    <script type="text/javascript">' . PHP_EOL;
        $template .= '        document.getElementById(\'%s\').addEventListener(\'change\', function(e) {' . PHP_EOL;
        $template .= '            if (e.target.files[0]) {' . PHP_EOL;
        $template .= '                var fileName          = e.target.files[0].name;' . PHP_EOL;
        $template .= '                var nextSibling       = e.target.nextElementSibling;' . PHP_EOL;
        $template .= '                nextSibling.innerText = fileName' . PHP_EOL;
        $template .= '            }' . PHP_EOL;
        $template .= '        });' . PHP_EOL;
        $template .= '    </script>' . PHP_EOL;
        $template .= '</div>';

        $this->class = str_replace('form-control', 'custom-file-input', $this->class);
        $this->type = 'file';

        $render = vsprintf($template, [
            $this->renderInput(),
            $this->id,
            $this->placeholder,
            $this->id,
        ]);

        return $render;
    }

    /**
     * Retourne le rendu du champs PDF avec ses paramètres
     * @return string Le rendu du champs
     */
    public function renderPdf(): string
    {
        $template  = '<div class="input-group">' . PHP_EOL;
        $template .= '    <div class="input-group-prepend">' . PHP_EOL;
        $template .= '        <div class="input-group-text text-danger">' . PHP_EOL;
        $template .= '            <i class="fa fa-file-pdf-o" aria-hidden="true"></i>' . PHP_EOL;
        $template .= '        </div>' . PHP_EOL;
        $template .= '    </div>' . PHP_EOL;
        $template .= '    %s' . PHP_EOL;
        $template .= '    <div class="input-group-append">' . PHP_EOL;
        $template .= '        <button class="btn btn-outline-secondary popupfinder" type="button" id="btn-%s" data-field="%s" data-mimes="%s">' . PHP_EOL;
        $template .= '            %s' . PHP_EOL;
        $template .= '        </button>' . PHP_EOL;
        $template .= '    </div>' . PHP_EOL;
        $template .= '</div>';

        $render = vsprintf($template, [
            $this->renderInput(),
            $this->id,
            '#' . $this->id,
            'application/pdf',
            $this->suffix ?? '<i class="fa fa-file-pdf-o mr-2"></i>Choisir PDF'
        ]);

        $this->prefix = null;
        $this->suffix = null;

        return $render;
    }

    /**
     * Retourne le rendu du champs Video avec ses paramètres
     * @return string Le rendu du champs
     */
    public function renderVideo(): string
    {
        $template  = '<div class="input-group">' . PHP_EOL;
        $template .= '    <div class="input-group-prepend">' . PHP_EOL;
        $template .= '        <div class="input-group-text">' . PHP_EOL;
        $template .= '            <i class="fa fa-video-camera" aria-hidden="true"></i>' . PHP_EOL;
        $template .= '        </div>' . PHP_EOL;
        $template .= '    </div>' . PHP_EOL;
        $template .= '    %s' . PHP_EOL;
        $template .= '    <div class="input-group-append">' . PHP_EOL;
        $template .= '        <button class="btn btn-outline-secondary popupfinder" type="button" id="btn-%s" data-field="%s" data-mimes="%s">' . PHP_EOL;
        $template .= '            %s' . PHP_EOL;
        $template .= '        </button>' . PHP_EOL;
        $template .= '    </div>' . PHP_EOL;
        $template .= '</div>';

        $render = vsprintf($template, [
            $this->renderInput(),
            $this->id,
            '#' . $this->id,
            'video',
            $this->suffix ?? '<i class="fa fa-file-video-o mr-2"></i>Choisir vidéo'
        ]);

        $this->prefix = null;
        $this->suffix = null;

        return $render;
    }

    /**
     * Retourne le rendu du champs Image avec ses paramètres
     * @return string Le rendu du champs
     */
    public function renderFolder(): string
    {
        $template  = '<div class="input-group img-group">' . PHP_EOL;
        $template .= '    <div class="input-group-prepend">' . PHP_EOL;
        $template .= '        <img id="thumb-%s" class="image-thumb" src="%s" alt="thumb">' . PHP_EOL;
        $template .= '    </div>' . PHP_EOL;
        $template .= '    %s' . PHP_EOL;
        $template .= '    <div class="input-group-append">' . PHP_EOL;
        $template .= '        <button class="btn btn-outline-secondary popupfinder" type="button" id="btn-%s" data-field="%s" data-folder="true">' . PHP_EOL;
        $template .= '            %s' . PHP_EOL;
        $template .= '        </button>' . PHP_EOL;
        $template .= '    </div>' . PHP_EOL;
        $template .= '</div>';

        $render = vsprintf($template, [
            $this->id,
            url('images/folder.jpg'),
            $this->renderInput(),
            $this->id,
            '#' . $this->id,
            $this->suffix ?? '<i class="fa fa-picture-o mr-2"></i>Choisir image'
        ]);

        $this->prefix = null;
        $this->suffix = null;

        return $render;
    }

    /**
     * Retourne le rendu du champs Image avec ses paramètres
     * @return string Le rendu du champs
     */
    public function renderImage(): string
    {
        $template  = '<div class="input-group img-group">' . PHP_EOL;
        $template .= '    <div class="input-group-prepend">' . PHP_EOL;
        $template .= '        <img id="thumb-%s" class="image-thumb" src="%s" alt="thumb">' . PHP_EOL;
        $template .= '    </div>' . PHP_EOL;
        $template .= '    %s' . PHP_EOL;
        $template .= '    <div class="input-group-append">' . PHP_EOL;
        $template .= '        <button class="btn btn-outline-secondary popupfinder" type="button" id="btn-%s" data-thumb="%s" data-field="%s" data-mimes="%s">' . PHP_EOL;
        $template .= '            %s' . PHP_EOL;
        $template .= '        </button>' . PHP_EOL;
        $template .= '    </div>' . PHP_EOL;
        $template .= '</div>';

        $render = vsprintf($template, [
            $this->id,
            empty($this->getValue()) ? url('images/default.svg') : url($this->getValue()),
            $this->renderInput(),
            $this->id,
            '#thumb-' . $this->id,
            '#' . $this->id,
            'image',
            $this->suffix ?? '<i class="fa fa-picture-o mr-2"></i>Choisir image'
        ]);

        $this->prefix = null;
        $this->suffix = null;

        return $render;
    }

    /**
     * Retourne le rendu du champs Toogle avec ses paramètres
     * @return string Le rendu du champs toggle
     */
    public function renderToggle(): string
    {
        $availables = ['id', 'class', 'name', 'is_disabled', 'is_required', 'is_readonly', 'style'];

        $template  = '<div class="toogle">' . PHP_EOL;
        $template .= '    <input type="checkbox" value="1" %s%s />' . PHP_EOL;
        $template .= '</div>';

        $render = vsprintf($template, [
            $this->renderFieldAttrs($availables),
            $this->getValue() == 1 ? ' checked' : ''
        ]);

        return $render;
    }

    /**
     * Retourne le rendu du champs radio avec ses paramètres
     * @return string Le rendu du champs radio
     */
    public function renderRadio(): string
    {
        $this->is_multiple = false;
        $this->class = str_replace('form-control', 'form-check-input', $this->class);

        return $this->renderCheckedOptions('radio');
    }

    /**
     * Retourne le rendu du champs checkbox avec ses paramètres
     * @return string Le rendu du champs checkbox
     */
    public function renderCheckbox(): string
    {
        $this->is_multiple = count($this->_options) > 1;
        $this->class = str_replace('form-control', 'form-check-input', $this->class);

        return $this->renderCheckedOptions('checkbox');
    }

    /**
     * Retourne le rendu du champs range avec ses paramètres
     * @return string Le rendu du champs range
     */
    public function renderRange(): string
    {
        $availables = ['id', 'class', 'name', 'value', 'min', 'max', 'is_disabled', 'ris_equired', 'is_readonly', 'style'];

        $functionName = Str::camel($this->name) . 'ShowValue';
        $template  = '<input type="range" %s oninput="%s(this.value)" />' . PHP_EOL;
        $template .= '<span id="%s">%s&nbsp;</span>'. PHP_EOL;
        $template .= '<script type="text/javascript">' . PHP_EOL;
        $template .= '    function %s(newValue) {' . PHP_EOL;
        $template .= '        document.getElementById("%s").innerHTML=newValue;' . PHP_EOL;
        $template .= '    }' . PHP_EOL;
        $template .= '</script>';

        $render = vsprintf($template, [
            $this->renderFieldAttrs($availables),
            $functionName,
            $this->id . '-value',
            $this->value,
            $functionName,
            $this->id . '-value'
        ]);

        return $render;
    }


    /**
     * Retourne le rendu du champs textarea avec ses paramètres
     * @return string Le rendu du champs textarea
     */
    public function renderTextarea(): string
    {
        $availables = ['id', 'class', 'name', 'placeholder', 'is_autocomplete', 'is_autofocus', 'is_disabled', 'is_required', 'is_readonly', 'rows', 'style'];

        $template = '<textarea %s>%s</textarea>';

        $render = vsprintf($template, [
            $this->renderFieldAttrs($availables),
            str_replace("\r\n", '[CR]', $this->getValue())
        ]);

        return $render;
    }

    /**
     * Retourne le rendu du champs select avec ses paramètres;
     * @return string Le rendu du champs select
     */
    public function renderSelect(): string
    {
        $availables = ['id', 'class', 'name', 'is_multiple', 'is_disabled', 'is_required', 'is_readonly', 'style'];

        $template  = '<select %s>' . PHP_EOL;
        $template .= '%s';
        $template .= '</select>';

        $render = vsprintf($template, [
            $this->renderFieldAttrs($availables),
            $this->renderIndent($this->renderOptions(), 1),
        ]);

        return $render;
    }

    /**
     * Retourne le rendu du champs input avec ses paramètres;
     * @return string Le rendu du champs input
     */
    public function renderInput(): string
    {
        $availables = ['id', 'type', 'class', 'name', 'value', 'placeholder', 'is_autocomplete', 'is_autofocus', 'is_disabled', 'is_required', 'is_readonly', 'style'];

        $template = '<input %s />';

        if (is_null($this->type)) {
            $this->type = 'text';
        }

        $render = sprintf($template, $this->renderFieldAttrs($availables));

        return $render;
    }

    /**
     * Retourne le rendu d'un champs avec son environnement
     * @param  string $renderField Le rendu du champs
     * @return string              Le rendu de l'environnement du champs
     */
    public function renderWrapFieldEnvironment(string $renderField, $noindentfield = false): string
    {
        $render   = $renderField;
        $template = '';

        $templateInline  = '<div class="form-group row">' . PHP_EOL;
        $templateInline .=     '%s%s';
        $templateInline .= '    <div class="col-sm-10">' . PHP_EOL;
        $templateInline .=         '%s';
        $templateInline .= '    </div>' . PHP_EOL;
        $templateInline .= '</div>' . PHP_EOL;

        $templateClassic  = '<div class="form-group">' . PHP_EOL;
        $templateClassic .=     '%s%s' ;
        $templateClassic .=     '%s';
        $templateClassic .= '</div>' . PHP_EOL;

        $templatePrefix  = '<div class="input-group%s">'. PHP_EOL;
        $templatePrefix .=     '%s%s%s%s';
        $templatePrefix .= '</div>'. PHP_EOL;

        // Si prefixe ou suffixe
        if (!empty($this->prefix) || !empty($this->suffix)) {
            $class_size = empty($this->size) ? '' : ' input-group-' . $this->size;

            $render = vsprintf($templatePrefix, [
                $class_size,
                $this->renderIndent($this->renderPrefix(), 1),
                $this->renderIndent($render, $noindentfield ? 0 : 1),
                $this->renderIndent($this->renderSuffix(), 1),
                $this->renderValidationMessage()
            ]);

            $render .= $this->renderHelp();
        } else {

            $render .= $this->renderHelp() . $this->renderValidationMessage();
        }


        // Affichage en mode inline
        if ($this->is_inline && ($this->type != 'checkbox' || $this->type != 'radio')) {
            $render = $this->renderIndent($render, $noindentfield ? 0 : 2);

            $render = vsprintf($templateInline, [
                $this->renderIndent($this->renderLabel(), 1),
                $this->renderIndent($this->renderPopover(), 1),
                $render,
            ]);
        } else {
            $render = $this->renderIndent($render, $noindentfield ? 0 : 1);

            $render = vsprintf($templateClassic, [
                $this->renderIndent($this->renderLabel(), 1),
                $this->renderIndent($this->renderPopover(), 1),
                $render,
            ]);
        }

        return $render;
    }

    /**
     * Retourne le rendu d'une chaine de caractère en HTML
     * @param  String $string La chains de caractères à traiter
     * @return String         Le rendu HTML
     */
    public function renderHTML($string): HtmlString
    {
        return new HtmlString($string);
    }

    /**
     * Retourne le rendu de la partie aide d'un champs (sous le champs)
     * @return string Le rendu de la partie aide
     */
    private function renderHelp(): string
    {
        $render = '';

        if (!empty($this->help)) {
            $template = '<small class="form-text text-muted text-right mt-0">%s</small>' . PHP_EOL;
            $render   = sprintf($template, $this->help);
        }

        return $render;
    }

    /**
     * Retoune le rendu de la partie prefixe du champs
     * @return string Le rendu du prefixe
     */
    private function renderPrefix(): string
    {
        $render = '';

        if (!empty($this->prefix)) {
            $template  = '<div class="input-group-prepend">' . PHP_EOL;
            if (preg_match('/<button.*/', $this->prefix)) {
                $template .= '    %s' . PHP_EOL;
            } else {
                $template .= '    <div class="input-group-text">%s</div>' . PHP_EOL;
            }
            $template .= '</div>' . PHP_EOL;

            $render = sprintf($template, $this->prefix);
        }
        return $render;
    }

    /**
     * Retourne le rendu de la partie suffixe du champs
     * @return string Le rendu du suffixe
     */
    private function renderSuffix(): string
    {
        $render = '';

        if (!empty($this->suffix)) {
            $template  = '<div class="input-group-append">' . PHP_EOL;
            if (preg_match('/<button.*/', $this->suffix)) {
                $template .= '    %s' . PHP_EOL;
            } else {
                $template .= '    <div class="input-group-text">%s</div>' . PHP_EOL;
            }
            $template .= '</div>' . PHP_EOL;

            $render = sprintf($template, $this->suffix);
        }

        return $render;
    }

    /**
     * Retourne le rendu de la popover (description du champs)
     * @return string Le rendu de la popover
     */
    private function renderPopover(): string
    {
        $render = '';

        if (!empty($this->_popover)) {
            $template  = '<span class="text-info popup-help" data-toggle="popover" data-placement="%s" title="%s" data-content="%s">' . PHP_EOL;
            $template .= '    <i class="fa fa-info-circle"></i>' . PHP_EOL;
            $template .= '</span>' . PHP_EOL;

            $render = vsprintf($template, [
                data_get($this->_popover, 'placement', 'right'),
                data_get($this->_popover, 'title', 'Informations'),
                data_get($this->_popover, 'content'),
            ]);
        }


        return $render;
    }

    /**
     * Retourne le rendu du label du champs
     * @return string Le rendu du label
     */
    private function renderLabel(): string
    {
        $render = '';
        $class  = '';

        if ($this->is_required) {
            $class .= 'required';
        }

        if ($this->is_inline && ($this->type != 'checkbox' || $this->type != 'radio')) {
            $class .= 'col-sm-2';
        }

        if (!empty($this->label)) {
            $template = '<label for="%s"%s>%s</label>' . PHP_EOL;

            $render = vsprintf($template, [
                $this->id,
                strlen($class) > 0 ? ' class="' . $class . '"' : '',
                $this->label
            ]);
        }


        return $render;
    }

    /**
     * Rendu d'optiond de type checked
     *
     * @param string $type type de champs à rendre
     * @return string Le rendu du champs
     */
    private function renderCheckedOptions(string $type = 'checkbox')
    {
        $availables = ['id', 'class', 'name', 'value', 'is_disabled', 'is_required', 'is_readonly', 'style'];

        $template  = '<div class="form-check%s%s">' . PHP_EOL;
        $template .= '    <input type="%s" %s%s%s/>' . PHP_EOL;
        $template .= '    <label class="form-check-label" for="%s">%s</label>' . PHP_EOL;
        $template .= '</div>' . PHP_EOL;

        $render = "";

        $current_value = old($this->name, $this->is_multiple ? $this->_values : $this->value);

        $options = $this->optionsModeling();

        foreach ($options as $option) {

            // Gestion de la valeur
            $this->value = $option->getOptionValue();
            if (is_array($current_value)) {
                $checked = in_array($this->value, $current_value);
            } else {
                $checked = $this->value == $current_value;
            }

            // Gestion des ID
            if (!is_bool($this->indice)) {
                $this->id = Str::slug($this->name .  '-' . $this->indice . '-' . $this->value);
            } else {
                $this->id = Str::slug($this->name .  '-' . $this->value);
            }


            $render .= vsprintf($template, [
                $this->is_disabled ? ' disabled' : '',
                $this->is_inline ? ' form-check-inline' : '',
                $type,
                $this->renderFieldAttrs($availables),
                $option->getDatasRender(),
                $checked ? ' checked' : '',
                $this->id,
                $option->getOptionLabel()
            ]);
        }


        return $render;
    }

    /**
     * Modélisation des options
     * Collection d'Option modélisées
     */
    private function optionsModeling()
    {
        $options = collect();

        if (is_array($this->_options)) {
            $this->checkOptionsValue();

            if (isset($this->_options['collect'])) {

                foreach ($this->_options['collect'] as $key => $item) {

                    $option = new Option();
                    $option->setSource($item);
                    $option->setLabelParam(data_get($this->_options, 'text'));
                    $option->setValueParam(data_get($this->_options, 'value'));
                    $option->setDatas(data_get($this->_options, 'datas'));
                    $option->setClass(data_get($this->_options, 'class'));
                    $option->setTemplate(data_get($this->_options, 'template'));
                    $option->setCallback(data_get($this->_options, 'callback'));
                    $option->setAttributes(data_get($this->_options, 'attributes'));
                    $options->push($option);
                }

            } else {
                foreach ($this->_options as $key => $item) {
                    $option = new Option();
                    $option->setSource([$key => $item]);
                    $options->push($option);

                }
            }
        }

        return $options;
    }

    /**
     * Retourne le rendu d'options d'un menu déroulant select
     * @return string Le rendu des options
     */
    private function renderOptions(): string
    {
        $render   = '';
        $template = '<option value="%s"%s>%s</option>' . PHP_EOL;
        $template_group = '<optgroup label="%s">%s</optgroup>' . PHP_EOL;
        $default  = '<option value=""' . ($this->is_required ? ' disabled hidden' : '') . ' selected >%s</option>' . PHP_EOL;

        if (!is_null($this->placeholder)) {
            $render = sprintf($default, $this->placeholder);
        }

        // Détection d'une collection source
        if (isset($this->_options['collect'])) {

            $this->checkOptionsValue();

            foreach ($this->_options['collect'] as $key => $item) {

                $attrs = "";

                // L'élément courant correspond à la valeur du champs
                $field_name  = is_int($this->indice) ? $this->name . '.' . $this->indice : $this->name;
                $field_value = old($field_name, $this->is_multiple ? $this->_values : $this->value);
                $item_value  = data_get($item, $this->_options['value']);

                // Attributs data-*
                if (isset($this->_options['datas']) && is_array($this->_options['datas'])) {
                    foreach ($this->_options['datas'] as $name => $value) {
                        $attrs .= $this->renderDatas([$name => data_get($item, $value)]);
                    }
                    $attrs = ' ' . trim($attrs);

                }

                if ((is_array($field_value) && in_array($item_value, $field_value)) ||
                    (!is_array($field_value) && strval($field_value) === strval($item_value))) {
                    $attrs .= ' selected';
                }

                $text     = data_get($this->_options, 'text');
                $callback = data_get($this->_options, 'callback');

                // Gestion d'un template
                if (isset($this->_options['template'])) {

                    // 1ère passe : ajout d'un template pour l'affichage du texte de l'option
                    $render .= vsprintf($template, [
                        data_get($item, $this->_options['value']),
                        $attrs,
                        $this->objectToString($this->_options['template'], $text, $callback, $item)
                    ]);

                } else {

                    $render .= vsprintf($template, [
                        data_get($item, $this->_options['value']),
                        $attrs,
                        $this->objectToString(null, $text, $callback, $item)
                    ]);
                }
            }

        } else {

            foreach ($this->_options as $value => $text) {

                $opt_group = '';
                // Cas d'un sous niveau
                if (is_array($text)) {
                    foreach ($text as $value_ => $text_) {
                        if (empty($this->name)) {
                            $attrs = strval($this->value) === strval($value_) ? ' selected' : '';
                        } else {
                            $attrs = strval(old($this->name, $this->value)) === strval($value_) ? ' selected' : '';
                        }

                        $opt_group .= vsprintf($template, [
                            $value_,
                            $attrs,
                            $text_
                        ]);
                    }

                    $render .= vsprintf($template_group, [
                        $value,
                        $opt_group
                    ]);
                } else {

                    if (empty($this->name)) {
                        $attrs = strval($this->value) === strval($value) ? ' selected' : '';
                    } else {
                        $attrs = strval(old($this->name, $this->value)) === strval($value) ? ' selected' : '';
                    }

                    $render .= vsprintf($template, [
                        $value,
                        $attrs,
                        $text
                    ]);
                }
            }
        }

        return $render;
    }

    /**
     * Retourne le rendu d'un message d'erreur de validation
     * @return string Le rendu du message
     */
    private function renderValidationMessage()
    {
        $errors = session('errors', collect());

        $render = null;

        if ($errors->has($this->name)) {
            $template = '<div class="invalid-feedback">%s</div>' . PHP_EOL;

            $render = sprintf($template, $errors->first($this->name));
        }

        return $render;
    }

    /**
     * Retourne le rendu des attributs d'un champs
     * @param  array  $availables Liste des attributs pouvant composer le champs
     * @return string             Le rendu des attributs
     */
    private function renderFieldAttrs(array $availables, $fixvalue = false): string
    {
        $attrs = get_object_vars($this);

        $render            = '';
        $template          = '%s="%s" ';
        $except_empty_attr = ['value'];

        foreach ($attrs as $attr => $value) {

            // On traite uniquement les attributs autorisés
            if (in_array($attr, $availables)) {

                // On ne traite pas les attributs avec une valeur vide
                if (!in_array($attr, $except_empty_attr) && empty($value)) {
                    continue;
                }

                switch ($attr) {
                    case 'value':
                        if ($fixvalue) {
                            $val = $this->value;
                        } else {
                            $val = $this->getValue();
                        }
                        break;
                    case 'class':
                        $val = $this->getClasses();
                        break;
                    case 'name' :
                        $val = $value;

                        // Mode indice
                        if (!is_null($this->indice)) {
                            $val .= '['. ($this->indice !== true ? $this->indice : '') .']';
                        }
                        if ($this->is_multiple) {
                            $val .= '[]';
                        }
                        break;
                    default:
                        if (is_bool($value)) {
                            $attr = str_replace('is_', '', $attr);
                            $val = str_replace('is_', '', $attr);
                        } else {
                            $val = $value;
                        }
                        break;
                }

                $render .= vsprintf($template, [$attr, $val]);
            }
        }

        // Attributs supplémentaires
        if (!is_null($this->_attributes) && is_array($this->_attributes)) {
            foreach ($this->_attributes as $attr => $value) {

                if (is_array($value)) {
                    $value_ = $this->objectToString(
                        data_get($value, 'template'),
                        data_get($value, 'attributes'),
                        data_get($value, 'callback'),
                        data_get($value, 'source')
                    );
                } else {
                    $value_ = $value;
                }
                $render .= vsprintf($template, [$attr, $value_]);
            }
        }

        // Ajout des datas
        $render .= $this->renderDatas($this->_datas);

        return trim($render);
    }

    /**
     * Retourne le rendu avec indentation d'un texte
     * @param  string $text  Le texte à traiter
     * @param  Integer $nb   Le nombre d'indentation à appliquer
     * @return string        Le rendu avec indentation
     */
    private function renderIndent($text, $nb): string
    {
        $indent = '    ';
        $render = '';

        $real_indent = '';

        for ($i = 1; $i <= $nb; $i++) {
            $real_indent .= $indent;
        }

        $lines = explode(PHP_EOL, $text);

        foreach ($lines as $line) {

            if (!empty($line)) {
                $render .= $real_indent . $line . PHP_EOL;
            }
        }

        return $render;
    }

    /**
     * Retourne le rendu des attributs datas
     * @param  array $datas   La liste des attributs data
     * @return string         Le rendu avec les attributs data
     */
    protected function renderDatas($datas): string
    {
        $render = '';

        if (!empty($datas) && is_array($datas)) {
            $template = 'data-%s="%s" ';
            foreach ($datas as $name => $value) {
                if (is_array($value)) {
                    $value_ = $this->objectToString(
                        data_get($value, 'template'),
                        data_get($value, 'attributes'),
                        data_get($value, 'callback'),
                        data_get($value, 'source')
                    );
                } else {
                    $value_ = $value;
                }
                $render .= vsprintf($template, [$name, $value_]);
            }
        }

        return $render;
    }

    /**
     * Retourne la liste de toutes les classes du champs
     * @return string Les classes du champs
     */
    private function getClasses(): string
    {
        $classes = $this->class . ' ' . $this->getValidationClass();

        // Si une taille est spécifiée
        if (!empty($this->size)) {

            // Aucun préfixe ou suffixe
            if (empty($this->prefix) && empty($this->suffix)) {
                $classes .= ' ' . 'form-control-' . $this->size;
            }
        }

        return trim($classes);
    }

    /**
     * Retourne la classe Bootstrap de validation du champs
     * @return string La classe
     */
    private function getValidationClass()
    {
        $errors = session('errors', collect());
        $class  = null;

        if ($errors->has($this->name)) {
            $class = 'is-invalid';
        } else if (!is_null($this->name) && !is_null(old($this->name))) {
            $class = 'is-valid';
        }

        return $class;
    }

    /**
     * Vérifie la cohérence de l'attribut Options
     * @return void
     */
    private function checkOptionsValue()
    {
        if (isset($this->_options['collect'])) {

            // Exceptions sur les paramètres
            if (is_array($this->_options['text']) && empty($this->_options['template'])) {
                throw new \InvalidArgumentException("Le paramètre options['template'] doit être présent lorsque options['text'] est un tableau");
            }
            if (isset($this->_options['template']) && !is_array($this->_options['text'])) {
                throw new \InvalidArgumentException("Le paramètre options['text'] doit être un tableau lorsque options['template'] est présent");
            }
        }

    }

    /**
     * Retourne une valeur depuis old ou l'attribut
     * @return string La valeur
     */
    private function getValue()
    {
        $name = $this->name;

        if (is_null($name)) {
            return ;
        }

        if (!is_null($this->indice)) {
            $name .= '.' . $this->indice;
        }
        return old($name, $this->value);
    }

    /**
     * Retourne le rendu d'un template avec une liste de noms de colonnes
     * @param  string $template         Le template servant au rendu
     * @param  string|array $display    Le ou les noms de colonnes
     * @param  string $callback         Fonction à appliquer sur la résultante
     * @param  Mix $item                Objet source des donnés à afficher
     * @param  Mix $item                Suppression des tags
     * @return string                   Le rendu
     */
    protected function objectToString($template = null, $displays = null, $callback = null, $item, $strip = false)
    {
        // Création du texte à afficher
        $datas = [];

        // Convertion en tableau si besoin
        if (!is_iterable($displays)) {
           $displays = [$displays];
        }

        // Traitement du tableau de valeur à afficher
        foreach ($displays as $key => $display) {

            $data = mixte_get($item, $display);

            // Callback array
            if (is_array($callback) && isset($callback[$key]) && !is_null($callback[$key])) {
                $data = $callback[$key]($data);
            }

            $datas[] = $strip ? strip_tags($data) : $data;
        }

        // Traitement des % seuls pour éviter les conflits
        if (!empty($template)) {
            $template = preg_replace('/(%)[^0-9sd.\']/i', '%%"', $template);
        }

        // Traitement template & callback
        if (is_string($callback)) {
            if (empty($template)) {
                $string = call_user_func_array($callback, $datas);
            } else {
                $string = vsprintf($template, $datas);
                $string = $callback($string);
            }

        } else if (!empty($template)) {
            $string = vsprintf($template, $datas);
        } else {
            $string = $datas[0];
        }

        return $string;
    }

    /**
     * Mise à jour des attributs depuis le tableau passé en paramètre
     * @param Array $attrs Liste des attributs à traiter
     * @return void
     */
    private function setAttrs($attrs)
    {
        $this->resetAttrs();

        if (is_null($attrs)) {
            return ;
        }

        // Liste des attributs autorisés
        $attrs_available = array_map(function ($val) {
            return trim($val, '_');
        }, array_keys(get_object_vars($this)));

        // Traitement des attributs
        if (is_array($attrs)) {
            foreach ($attrs as $attr => $value) {
                if (in_array($attr, $attrs_available)) {
                    if (is_array($value)) {
                        $this->{'_' . $attr} = $value;
                    } else {
                        $this->{$attr} = $value;
                    }
                } else {
                    throw new \InvalidArgumentException("Le paramètre [" . $attr . "] n'est pas pris en charge");
                }
            }
        } else {
            throw new \InvalidArgumentException("Type de paramètre invalide. Attendu : Tableau");
        }

        // Traitement de l'ID
        if (is_null($this->id)) {
            $this->id = 'field-' . $this->name;
        }
        if (!is_null($this->indice)) {
            $this->id = $this->id . '-' . $this->indice;
        }
        $this->id = Str::slug($this->id);

        // Traitement de la classe
        $this->class = 'form-control ' . $this->class;
        $this->class = trim($this->class);

        // Traitement sur la valeur
        if (is_null($this->value)) {
            $this->value = '';
        }

        // Traitement sur la taille
        if (!empty($this->size) && !in_array($this->size, ['sm', 'lg'])) {
             throw new \InvalidArgumentException("La valeur pour le paramètre « size » est invalide");
        }
    }

    /**
     * Réinitialisation des attributs de constructions
     * @return void
     */
    private function resetAttrs()
    {
        $attrs = array_keys(get_object_vars($this));

        foreach ($attrs as $attr) {

            if (preg_match('/^_.*/', $attr)) {
                $this->{$attr} = [];
            } else if (preg_match('/^is_.*/', $attr)) {
                $this->{$attr} = false;
            } else {
                $this->{$attr} = null;
            }
        }
    }
}
